local json = require("json")
local utils = require("utils")
local const = require("const")
local Set = require("set")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")
local pl_pretty = require("pl.pretty")

local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("TSLManage")

local function get_standard_tsl(pk)
    _, msg = utils.get_data(const.data_key.tsl.standard_tsl_prefix..pk)
    
    if (msg == nil) then
        msg = "{}"
    end

    return msg
end

local function get_extend_tsl(pk)
    _, msg = utils.get_data(const.data_key.tsl.extend_tsl_prefix..pk)
    
    if (msg == nil) then
        msg = "{}"
    end
    return msg
end

local function get_product_name_from_TSL(tsl)
    local args = json.decode(tsl) 
    if args then
        if args["profile"] then
            return args["profile"]["productName"]
        end
    else
        return nil
    end
end

if "POST" == request_method then
   local utils_header = require("utils_header")
   local params = utils_header.check_public_boby_data()
   
    local productKeyList = {}
    local productKey = params["productKey"]
    local pageSize = tonumber(params["PageSize"])
    local curPage = tonumber(params["CurrentPage"])
   
    pageSize = utils.check_page_size_allow_zero(pageSize)
    utils.check_current_page(curPage)
  
    if (productKey == nil) or (type(productKey) ~= "string") or #productKey == 0 then
        productKeyList = json.decode(utils.get_standard_tsl_key_list()) 
    else
        utils.check_pk(productKey)
        table.insert(productKeyList, const.data_key.tsl.standard_tsl_prefix..productKey) 
    end

    local list_data = {}
    local totalNum = #productKeyList
    if pageSize <= 0 then
        pageSize = totalNum
    end
    list_data["TotalNum"] = totalNum
    list_data["PageSize"] = pageSize
    list_data["CurrentPage"] = curPage

    local list_data_tsl = {}
   
    table.sort(productKeyList)
    local min, max = utils.get_item_index(totalNum, pageSize, curPage)
    for n = min, max do
        local key = productKeyList[n]
        local pk = string.sub(key, 1 + #const.data_key.tsl.standard_tsl_prefix)
        local list_data_tsl_item = {}
        local tsl = get_standard_tsl(pk) 
        list_data_tsl_item["productKey"] = pk
        list_data_tsl_item["standardTSL"] = tsl
        list_data_tsl_item["productName"] = get_product_name_from_TSL(tsl)
        list_data_tsl_item["extendTSL"] = get_extend_tsl(pk) 
        
        table.insert(list_data_tsl, list_data_tsl_item)
    end
    list_data["list"] = list_data_tsl

    utils.error_exit(200, json.encode(list_data)) 
end
utils.error_exit(400, "unsupported request method")

------ EOF ------
