local json = require("json")
local utils = require("utils")
local const = require("const")
local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("MessageRoute")

if "POST" == request_method then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_boby_data()
    local pageSize = params["PageSize"]
    local currentPage = params["CurrentPage"]
   
    pageSize = utils.check_page_size_allow_zero(pageSize)
    utils.check_current_page(currentPage)

    local ok, msg_raw, err = utils.dbus_call_edge(const.subscribe.wkn,
                                        const.subscribe.obj,
                                        const.subscribe.inf,
                                        const.subscribe.interface_read_list)
    local msg_ret = {}
    if (ok == "ok" and msg_raw) then
        local msg_raw_json = json.decode(msg_raw[1])
        local content = msg_raw_json["content"]
        local ret = {}
        local min, max = utils.get_item_index(#content, pageSize, currentPage)
        for k, v in ipairs(content) do
            if k >= min and k <= max then
                local item = {}
                item["SourceType"] = v["source"]
                item["SourceData"] = v["data"]
                item["TopicFilter"] = v["topicFilter"]
                item["TargetType"] = v["target"]["type"]
                item["TargetData"] = v["target"]["context"]
                table.insert(ret, item)
            end
        end
        msg_ret["TotalNum"] = #content 
        msg_ret["PageSize"] = pageSize 
        msg_ret["CurrentPage"] = currentPage 
        msg_ret["List"] = ret
    else
        utils.error_exit(500, json.encode("failed to query: "..err))
    end 

    utils.error_exit(200, json.encode(msg_ret)) 

end
utils.error_exit(400, "unsupported request method")

