#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ====#====#====#====
# author = "jys"
# file: test.py
# time: 12/28/17 6:29 AM
# ====#====#====#====
import sys

sys.path.append("..")

from leda_python.mbus import MbusBase
import unittest
import threading
import logging
import json
import time
import sys
import os

g_cmdArgs = ''
_logger = logging.getLogger(__name__)
mbusObject = MbusBase("iot.app.demoTest")


def createThread(task, args=()):
    t = threading.Thread(target=task, args=args)
    t.setDaemon(True)
    t.start()


def dimu_stop():
    os.system(
        "ps -ef | grep \"ut_dimu\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>>/dev/null")


def dimu_start():
    dimu_stop()
    os.system("./startDimu.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def config_stop():
    os.system(
        "ps -ef | grep \"ut_config\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>>/dev/null")


def config_start():
    config_stop()
    os.system("./startConfig.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def driver_stop():
    os.system(
        "ps -ef | grep \"ut_driver\" | grep -v grep | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>> /dev/null")


class AppUnitTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        dimu_start()
        config_start()

        if (g_cmdArgs == 'no_coverage'):
            _logger.debug('no coverage')
            os.system("./startDriver.sh & 1>> /dev/null 2>> /dev/null")

        else:
            os.system("./startDriver.sh coverage & 1>> /dev/null 2>> /dev/null")
        time.sleep(10)

    @classmethod
    def tearDownClass(cls):
        dimu_stop()
        config_stop()
        driver_stop()

    # @unittest.skip('i dont test the case')
    def test_getAllDeviceList(self):
        interfaceHandle = mbusObject.getRemoteInterface("iot.driver.abc123",
                                                        "/iot/driver/abc123",
                                                        "iot.driver.abc123")

        retInfo = interfaceHandle.getDeviceList('')
        retInfo = json.loads(retInfo)

        deviceNum = retInfo["params"]["devNum"]
        deviceCloudId_list = retInfo["params"]["devList"]

        _logger.info("all device num: %d", deviceNum)
        for cloud_id in deviceCloudId_list:
            _logger.info("\tdevice cloud_id: %s", cloud_id)

        self.assertEqual(deviceNum, 60)

    # @unittest.skip('i dont test the case')
    def test_getOnlineDeviceList(self):
        interfaceHandle = mbusObject.getRemoteInterface("iot.driver.abc123",
                                                        "/iot/driver/abc123",
                                                        "iot.driver.abc123")

        retInfo = interfaceHandle.getDeviceList('online')
        retInfo = json.loads(retInfo)

        deviceNum = retInfo["params"]["devNum"]
        deviceCloudId_list = retInfo["params"]["devList"]

        _logger.info("online device num: %d", deviceNum)
        for cloud_id in deviceCloudId_list:
            _logger.info("\tonline device cloud_id: %s", cloud_id)

        self.assertEqual(deviceNum, 50)

    # @unittest.skip('i dont test the case')
    def test_getOfflineDeviceList(self):
        interfaceHandle = mbusObject.getRemoteInterface("iot.driver.abc123",
                                                        "/iot/driver/abc123",
                                                        "iot.driver.abc123")

        retInfo = interfaceHandle.getDeviceList('offline')
        retInfo = json.loads(retInfo)

        deviceNum = retInfo["params"]["devNum"]
        deviceCloudId_list = retInfo["params"]["devList"]

        _logger.info("offline device num: %d", deviceNum)
        for cloud_id in deviceCloudId_list:
            _logger.info("\toffline device cloud_id: %s", cloud_id)

        self.assertEqual(deviceNum, 10)

    # @unittest.skip('i dont test the case')
    def test_getDeviceListWithInvalidParam(self):

        interfaceHandle = mbusObject.getRemoteInterface("iot.driver.abc123",
                                                        "/iot/driver/abc123",
                                                        "iot.driver.abc123")

        retInfo = interfaceHandle.getDeviceList('line')
        retInfo = json.loads(retInfo)

        deviceNum = retInfo["params"]["devNum"]
        deviceCloudId_list = retInfo["params"]["devList"]

        _logger.info("device num: %d", deviceNum)
        for cloud_id in deviceCloudId_list:
            _logger.info("\tdevice cloud_id: %s", cloud_id)

        self.assertEqual(deviceNum, 0)

    # @unittest.skip('i dont test the case')
    def test_callMethod_ok(self):
        for i in range(0, 10):
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("method_1", json.dumps(inparam))
            retInfo = json.loads(retInfo)
            code = retInfo['params']["code"]
            self.assertEqual(code, 0)

    # @unittest.skip('i dont test the case')
    def test_callMethod_retInvalidParams(self):
        for i in range(10, 20):
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("method_1", json.dumps(inparam))
            retInfo = json.loads(retInfo)
            print('%s' % (retInfo))
            code = retInfo['params']["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_callMethod_noMethodName(self):
        for i in range(20, 30):
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("method_1", json.dumps(inparam))
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100000)

    # @unittest.skip('i dont test the case')
    def test_callMetod_retInvalidType(self):
        _logger.debug("#################test: callServices( ret invalid type)################")
        for i in range(30, 40):
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("method_1", json.dumps(inparam))
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_callMethod_retInvalidCode(self):
        for i in range(40, 50):
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("method_1", json.dumps(inparam))
            _logger.debug("\tcallServices retInfo: %s", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_callMethod_offlineDev(self):
        for i in range(50, 60):
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            try:
                interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
                retInfo = interfaceHandle.callServices("method_1", json.dumps(inparam))
                _logger.debug("\tcallServices retInfo: %s", retInfo)
                assertID = False
            except:
                _logger.debug("not find device  ...Ok")
                assertID = True

            self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_callMethod_invalidMethodName(self):
        print('')
        _logger.debug("#################test: invaild method ################")
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "params": {
                    "args1": "xxx",
                    "args2": "yyy"
                }
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("method_2", json.dumps(inparam))
            _logger.debug("\tcallServices retInfo: %s", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_get_ok(self):
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": ["key1", "key2"]
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 0)

    # @unittest.skip('i dont test the case')
    def test_get_retErrCode(self):
        for i in range(10, 20):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": ["key1", "key2"]
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_get_noGet(self):
        for i in range(20, 30):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": ["key1", "key2"]
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100000)

    # @unittest.skip('i dont test the case')
    def test_get_retTypeInvalid(self):
        for i in range(30, 40):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": ["key1", "key2"]
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_get_retInvalidCode(self):
        for i in range(40, 50):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": ["key1", "key2"]
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_get_offlineDev(self):
        for i in range(50, 60):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": ["key1", "key2"]
            }
            try:
                interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
                retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
                _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
                retInfo = json.loads(retInfo)
                code = retInfo["code"]
                assertID = False

            except:
                assertID = True

            self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_get_invalidInpatams(self):
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "params_1": ["key1", "key2"]
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tget retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_set_ok(self):
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": {
                    "key1": "value1",
                    "key2": "value2"
                }
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("set", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 0)

    # @unittest.skip('i dont test the case')
    def test_set_retErrCode(self):
        for i in range(10, 20):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn

            inparam = {
                "params": {
                    "key1": "value1",
                    "key2": "value2"
                }
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("set", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_set_noSetMethod(self):
        for i in range(20, 30):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "params": {
                    "key1": "value1",
                    "key2": "value2"
                }
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("set", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100000)

    # @unittest.skip('i dont test the case')
    def test_set_retTypeInvalid(self):
        for i in range(30, 40):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "params": {
                    "key1": "value1",
                    "key2": "value2"
                }
            }

            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("set", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_set_retInvalidCode(self):
        for i in range(40, 50):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "params": {
                    "key1": "value1",
                    "key2": "value2"
                }
            }

            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("set", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_set_offlineDev(self):
        for i in range(50, 60):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "params": {
                    "key1": "value1",
                    "key2": "value2"
                }
            }

            try:
                interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
                retInfo = interfaceHandle.callServices("set", json.dumps(inparam))
                _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
                retInfo = json.loads(retInfo)
                code = retInfo["code"]

                assertId = False
            except:
                assertId = True

            self.assertEqual(assertId, True)

    # @unittest.skip('i dont test the case')
    def test_set_inparamsInvalid(self):
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "key1": "value1",
                "key2": "value2"
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_set_inpatamsNull(self):
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", '')
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)

    # @unittest.skip('i dont test the case')
    def test_set_inparamsTypeInvalid(self):
        for i in range(0, 10):
            wkn = "iot.device.idabcdefghigk_deviceaabb" + str(i)
            objpath = "/iot/device/idabcdefghigk_deviceaabb" + str(i)
            interface = wkn
            inparam = {
                "key1": "value1",
                1: "value2"
            }
            interfaceHandle = mbusObject.getRemoteInterface(wkn, objpath, interface)
            retInfo = interfaceHandle.callServices("get", json.dumps(inparam))
            _logger.debug("\tset retInfo: %s  ...Ok", retInfo)
            retInfo = json.loads(retInfo)
            code = retInfo["code"]
            self.assertEqual(code, 100001)


if (__name__ == "__main__"):
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s-%(threadName)-9s-%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s')

    _logger.debug("app testing...")

    # avoid: throw except
    if (len(sys.argv) >= 2):
        g_cmdArgs = sys.argv[1]
        del sys.argv[1]

    # suite = unittest.TestSuite()#创建测试套件
    unittest.main(verbosity=2)
