#!/usr/bin/env python


'''
Pymodbus Synchronous Server Example
--------------------------------------------------------------------------

The synchronous server is implemented in pure python without any third
party libraries (unless you need to use the serial protocols which require
pyserial). This is helpful in constrained or old environments where using
twisted just is not feasable. What follows is an examle of its use:
'''
import threading
import time
import threading
import signal
import sys
import getopt
#---------------------------------------------------------------------------#
# import the various server implementations
#---------------------------------------------------------------------------#
from pymodbus.server.sync import StartTcpServer, ModbusTcpServer
from pymodbus.server.sync import StartUdpServer
from pymodbus.server.sync import StartSerialServer

from pymodbus.device import ModbusDeviceIdentification
from pymodbus.datastore import ModbusSequentialDataBlock
from pymodbus.datastore import ModbusSlaveContext, ModbusServerContext

from pymodbus.transaction import ModbusRtuFramer, ModbusSocketFramer
#---------------------------------------------------------------------------#
# configure the service logging
#---------------------------------------------------------------------------#
import logging
logging.basicConfig(format='[%(levelname)-7s]:%(threadName)s:%(asctime)s:%(filename)s:%(lineno)d| %(message)s')
log = logging.getLogger()
log.setLevel(logging.DEBUG)

ModbusTcpServer.allow_reuse_address = True
g_lock = threading.Lock()
g_serverHandle_dict = {}

# 函数名称： createThread
# 功能描述： 创建独立线程
# 输入参数：
# 	task： 任务
# 	args: 任务的参数
# 返 回 值： 无
def createThread(task, *args):

	t = threading.Thread(target=task, args = args)
	t.setDaemon(True)
	t.start()

#---------------------------------------------------------------------------#
# initialize your data store
#---------------------------------------------------------------------------#
# The datastores only respond to the addresses that they are initialized to.
# Therefore, if you initialize a DataBlock to addresses of 0x00 to 0xFF, a
# request to 0x100 will respond with an invalid address exception. This is
# because many devices exhibit this kind of behavior (but not all)::
#
#     block = ModbusSequentialDataBlock(0x00, [0]*0xff)
#
# Continuing, you can choose to use a sequential or a sparse DataBlock in
# your data context.  The difference is that the sequential has no gaps in
# the data while the sparse can. Once again, there are devices that exhibit
# both forms of behavior::
#
#     block = ModbusSparseDataBlock({0x00: 0, 0x05: 1})
#     block = ModbusSequentialDataBlock(0x00, [0]*5)
#
# Alternately, you can use the factory methods to initialize the DataBlocks
# or simply do not pass them to have them initialized to 0x00 on the full
# address range::
#
#     store = ModbusSlaveContext(di = ModbusSequentialDataBlock.create())
#     store = ModbusSlaveContext()
#
# Finally, you are allowed to use the same DataBlock reference for every
# table or you you may use a seperate DataBlock for each table. This depends
# if you would like functions to be able to access and modify the same data
# or not::
#
#     block = ModbusSequentialDataBlock(0x00, [0]*0xff)
#     store = ModbusSlaveContext(di=block, co=block, hr=block, ir=block)
#
# The server then makes use of a server context that allows the server to
# respond with different slave contexts for different unit ids. By default
# it will return the same context for every unit id supplied (broadcast
# mode). However, this can be overloaded by setting the single flag to False
# and then supplying a dictionary of unit id to context mapping::
#
#     slaves  = {
#         0x01: ModbusSlaveContext(...),
#         0x02: ModbusSlaveContext(...),
#         0x03: ModbusSlaveContext(...),
#     }
#     context = ModbusServerContext(slaves=slaves, single=False)
#
# The slave context can also be initialized in zero_mode which means that a
# request to address(0-7) will map to the address (0-7). The default is
# False which is based on section 4.4 of the specification, so address(0-7)
# will map to (1-8)::
#
#     store = ModbusSlaveContext(..., zero_mode=True)
#---------------------------------------------------------------------------#
# store = ModbusSlaveContext(
#     di = ModbusSequentialDataBlock(0, [17]*100),
#     co = ModbusSequentialDataBlock(0, [17]*100),
#     hr = ModbusSequentialDataBlock(0, [17]*100),
#     ir = ModbusSequentialDataBlock(0, [17]*100))
# context = ModbusServerContext(slaves=store, single=True)

#---------------------------------------------------------------------------#
# initialize the server information
#---------------------------------------------------------------------------#
# If you don't set this or any fields, they are defaulted to empty strings.
#---------------------------------------------------------------------------#
# identity = ModbusDeviceIdentification()
# identity.VendorName  = 'Pymodbus'
# identity.ProductCode = 'PM'
# identity.VendorUrl   = 'http://github.com/riptideio/pymodbus/'
# identity.ProductName = 'Pymodbus Server'
# identity.ModelName   = 'Pymodbus Server'
# identity.MajorMinorRevision = '1.0'

#---------------------------------------------------------------------------#
# run the server you want
#---------------------------------------------------------------------------#
# Tcp:
# StartTcpServer(context, identity=identity, address=("localhost", 5020))

# Udp:
#StartUdpServer(context, identity=identity, address=("localhost", 502))

# Ascii:
#StartSerialServer(context, identity=identity, port='/dev/pts/3', timeout=1)

# RTU:
# StartSerialServer(context, framer=ModbusRtuFramer, identity=identity, port='/dev/ptyp0', timeout=.005, baudrate=9600)

def modbusTask(ip, port, server_handleDict):
    global g_lock
    store = ModbusSlaveContext(
        di=ModbusSequentialDataBlock(0, [1] * 100),
        co=ModbusSequentialDataBlock(0, [1] * 100),
        hr=ModbusSequentialDataBlock(0, [0x6162] * 100),
        ir=ModbusSequentialDataBlock(0, [0x6162] * 100))
    context = ModbusServerContext(slaves=store, single=True)

    identity = ModbusDeviceIdentification()
    identity.VendorName = 'Pymodbus'
    identity.ProductCode = 'PM'
    identity.VendorUrl = 'http://github.com/riptideio/pymodbus/'
    identity.ProductName = 'Pymodbus Server'
    identity.ModelName = 'Pymodbus Server'
    identity.MajorMinorRevision = '1.0'

    #StartTcpServer(context, identity=identity, address=(ip, port))
    framer = ModbusSocketFramer
    server = ModbusTcpServer(context, framer, identity, address = (ip, port))
    g_lock.acquire()
    server_handleDict[str(port)] = server
    g_lock.release()
    server.serve_forever()


def SignalCallBack(signum, stack):
    print("receive a signal...")
    for key in g_serverHandle_dict:
        print (key)
        g_serverHandle_dict[key].shutdown()
        g_serverHandle_dict[key].server_close()
    sys.exit(0)


def modbusSerialTask(port):
    global g_lock
    store = ModbusSlaveContext(
        di=ModbusSequentialDataBlock(0, [1] * 100),
        co=ModbusSequentialDataBlock(0, [1] * 100),
        hr=ModbusSequentialDataBlock(0, [0x6162] * 100),
        ir=ModbusSequentialDataBlock(0, [0x6162] * 100))
    context = ModbusServerContext(slaves=store, single=True)

    identity = ModbusDeviceIdentification()
    identity.VendorName = 'Pymodbus'
    identity.ProductCode = 'PM'
    identity.VendorUrl = 'http://github.com/riptideio/pymodbus/'
    identity.ProductName = 'Pymodbus Server'
    identity.ModelName = 'Pymodbus Server'
    identity.MajorMinorRevision = '1.0'

    StartSerialServer(context, framer=ModbusRtuFramer, identity=identity, port=port, timeout=.1, baudrate=9600)
    # StartSerialServer(context, identity=identity, port=port, timeout=1)
    # framer = ModbusSocketFramer
    # server = ModbusTcpServer(context, framer, identity, address = (ip, port))
    # g_lock.acquire()
    # server_handleDict[str(port)] = server
    # g_lock.release()
    # server.serve_forever()

if (__name__ == "__main__"):

    serverHandle_dict = {}
    signal.signal(signal.SIGINT, SignalCallBack)
    signal.signal(signal.SIGTERM, SignalCallBack)
    for num in range(15020, 15021):
        createThread(modbusTask, '', num, g_serverHandle_dict)
    argv = sys.argv[1:]
    try:
        opts, args = getopt.getopt(argv, "hp:", ["port="])
        for opt, arg in opts:
            if opt == '-h':
                print('test.py -p <port>')
            elif opt in ("-p", "--port"):
                createThread(modbusSerialTask, arg)
    except Exception as e:
        pass

    try:
        while(1):
            time.sleep(1)

    except SystemExit:
        print('exit')
        for key in g_serverHandle_dict:
            g_serverHandle_dict[key].server_close()

